from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcpk8_s_pot_create_or_modify_host_path_volume_mount_tests: list[RuleTest] = [
    RuleTest(
        name="Pod With Suspicious Volume Mount Created",
        expected_result=True,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "some.user@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.pods.create",
                        "resource": "core/v1/namespaces/default/pods/test",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {"name": "test", "namespace": "default"},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "test",
                                "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                            },
                        ],
                        "volumes": [
                            {
                                "hostPath": {"path": "/var/lib/kubelet", "type": "DirectoryOrCreate"},
                                "name": "test-volume",
                            },
                        ],
                    },
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                },
                "resourceName": "core/v1/namespaces/default/pods/test",
                "response": {
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "test",
                                "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                            },
                        ],
                        "volumes": [
                            {
                                "hostPath": {"path": "/var/lib/kubelet", "type": "DirectoryOrCreate"},
                                "name": "test-volume",
                            },
                        ],
                    },
                    "status": {"phase": "Pending", "qosClass": "BestEffort"},
                },
            },
            "receiveTimestamp": "2024-02-16 11:48:43.531373988",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-16 11:48:22.742154000",
        },
    ),
    RuleTest(
        name="Pod With Non-Suspicious Volume Mount Created",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "some.user@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.pods.create",
                        "resource": "core/v1/namespaces/default/pods/test",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {"name": "test", "namespace": "default"},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "test",
                                "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                            },
                        ],
                        "volumes": [
                            {"hostPath": {"path": "/data", "type": "DirectoryOrCreate"}, "name": "test-volume"},
                        ],
                    },
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                },
                "resourceName": "core/v1/namespaces/default/pods/test",
                "response": {
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "test",
                                "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                            },
                        ],
                        "volumes": [
                            {"hostPath": {"path": "/data", "type": "DirectoryOrCreate"}, "name": "test-volume"},
                        ],
                    },
                    "status": {"phase": "Pending", "qosClass": "BestEffort"},
                },
            },
            "receiveTimestamp": "2024-02-16 11:48:43.531373988",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-16 11:48:22.742154000",
        },
    ),
    RuleTest(
        name="Pod Not Created",
        expected_result=False,
        log={
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "some.user@company.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "io.k8s.core.v1.pods.create",
                        "resource": "core/v1/namespaces/default/pods/test",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.create",
                "request": {
                    "@type": "core.k8s.io/v1.Pod",
                    "apiVersion": "v1",
                    "kind": "Pod",
                    "metadata": {"name": "test", "namespace": "default"},
                    "spec": {
                        "containers": [
                            {
                                "image": "nginx",
                                "imagePullPolicy": "Always",
                                "name": "test",
                                "volumeMounts": [{"mountPath": "/test", "name": "test-volume"}],
                            },
                        ],
                        "volumes": [
                            {
                                "hostPath": {"path": "/var/lib/kubelet", "type": "DirectoryOrCreate"},
                                "name": "test-volume",
                            },
                        ],
                    },
                    "status": {},
                },
                "resourceName": "core/v1/namespaces/default/pods/test",
                "response": {"status": "Failure"},
            },
            "receiveTimestamp": "2024-02-16 12:55:17.003485190",
            "resource": {
                "labels": {
                    "cluster_name": "some-project-cluster",
                    "location": "us-west1",
                    "project_id": "some-project",
                },
                "type": "k8s_cluster",
            },
            "timestamp": "2024-02-16 12:55:00.510160000",
        },
    ),
]


@panther_managed
class GCPK8SPotCreateOrModifyHostPathVolumeMount(Rule):
    id = "GCP.K8S.Pot.Create.Or.Modify.Host.Path.Volume.Mount-prototype"
    display_name = "GCP K8S Pot Create Or Modify Host Path Volume Mount"
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_description = "This detection monitors for pod creation with a hostPath volume mount. The attachment to a node's volume can allow for privilege escalation through underlying vulnerabilities or it can open up possibilities for data exfiltration or unauthorized file access. It is very rare to see this being a pod requirement.\n"
    default_runbook = "Investigate the reason of adding hostPath volume mount. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://linuxhint.com/kubernetes-hostpath-volumes/"
    reports = {"MITRE ATT&CK": ["TA0001", "TA0002"]}
    tests = gcpk8_s_pot_create_or_modify_host_path_volume_mount_tests
    SUSPICIOUS_PATHS = [
        "/var/run/docker.sock",
        "/var/run/crio/crio.sock",
        "/var/lib/kubelet",
        "/var/lib/kubelet/pki",
        "/var/lib/docker/overlay2",
        "/etc/kubernetes",
        "/etc/kubernetes/manifests",
        "/etc/kubernetes/pki",
        "/home/admin",
    ]

    def rule(self, event):
        if deep_get(event, "protoPayload", "response", "status") == "Failure":
            return False
        if deep_get(event, "protoPayload", "methodName") not in (
            "io.k8s.core.v1.pods.create",
            "io.k8s.core.v1.pods.update",
            "io.k8s.core.v1.pods.patch",
        ):
            return False
        volume_mount_path = deep_walk(event, "protoPayload", "request", "spec", "volumes", "hostPath", "path")
        if not volume_mount_path or (
            volume_mount_path not in self.SUSPICIOUS_PATHS
            and (not any(path in self.SUSPICIOUS_PATHS for path in volume_mount_path))
        ):
            return False
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if (
                auth.get("permission")
                in ("io.k8s.core.v1.pods.create", "io.k8s.core.v1.pods.update", "io.k8s.core.v1.pods.patch")
                and auth.get("granted") is True
            ):
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        pod_name = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created k8s pod [{pod_name}] with a hostPath volume mount in project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        volume_mount_path = deep_walk(event, "protoPayload", "request", "spec", "volumes", "hostPath", "path")
        context["volume_mount_path"] = volume_mount_path
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcpsql_config_changes_tests: list[RuleTest] = [
    RuleTest(
        name="Sql Instance Change",
        expected_result=True,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {"principalEmail": "user@runpanther.io"},
                "requestMetadata": {
                    "callerIp": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                    "requestAttributes": {"time": "2020-05-15T04:28:42.243082428Z", "auth": {}},
                    "destinationAttributes": {},
                },
                "serviceName": "storage.googleapis.com",
                "methodName": "cloudsql.instances.update",
            },
            "resource": {
                "type": "sql_instance",
                "labels": {"project_id": "western-verve-123456", "location": "asia-northeast2"},
            },
        },
    ),
]


@panther_managed
class GCPSQLConfigChanges(Rule):
    id = "GCP.SQL.ConfigChanges-prototype"
    display_name = "GCP SQL Config Changes"
    dedup_period_minutes = 720
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Database"]
    reports = {"CIS": ["2.11"]}
    default_severity = Severity.LOW
    default_description = "Monitoring changes to Sql Instance configuration may reduce time to detect and correct misconfigurations done on sql server.\n"
    default_runbook = "Validate the Sql Instance configuration change was safe"
    default_reference = "https://cloud.google.com/sql/docs/mysql/instance-settings"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    tests = gcpsql_config_changes_tests

    def rule(self, event):
        return deep_get(event, "protoPayload", "methodName") == "cloudsql.instances.update"

    def dedup(self, event):
        return deep_get(event, "resource", "labels", "project_id", default="<UNKNOWN_PROJECT>")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_cloud_trail_login_profile_created_or_modified_tests: list[RuleTest] = [
    RuleTest(
        name="ChangeOwnPassword",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "1234",
            "eventName": "UpdateLoginProfile",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-09-15 13:45:24",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "987654321",
            "requestParameters": {"passwordResetRequired": False, "userName": "alice"},
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ABC1234",
                "accountId": "987654321",
                "arn": "arn:aws:sts::98765432:assumed-role/IAM/alice",
                "principalId": "ABCDE:alice",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-15T13:36:47Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "987654321",
                        "arn": "arn:aws:iam::9876432:role/IAM",
                        "principalId": "1234ABC",
                        "type": "Role",
                        "userName": "IAM",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="User changed password for other",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "1234",
            "eventName": "UpdateLoginProfile",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-09-15 13:45:24",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "987654321",
            "requestParameters": {"passwordResetRequired": False, "userName": "bob"},
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ABC1234",
                "accountId": "987654321",
                "arn": "arn:aws:sts::98765432:assumed-role/IAM/alice",
                "principalId": "ABCDE:alice",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-15T13:36:47Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "987654321",
                        "arn": "arn:aws:iam::9876432:role/IAM",
                        "principalId": "1234ABC",
                        "type": "Role",
                        "userName": "IAM",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailLoginProfileCreatedOrModified(Rule):
    id = "AWS.CloudTrail.LoginProfileCreatedOrModified-prototype"
    display_name = "AWS User Login Profile Created or Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.LOW
    reports = {"MITRE ATT&CK": ["TA0003:T1098", "TA0005:T1108", "TA0005:T1550", "TA0008:T1550"]}
    default_description = "An attacker with iam:UpdateLoginProfile permission on other users can change the password used to login to the AWS console. May be legitimate account administration."
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_examples_aws_my-sec-creds-self-manage-pass-accesskeys-ssh.html"
    tests = aws_cloud_trail_login_profile_created_or_modified_tests
    PROFILE_EVENTS = {"UpdateLoginProfile", "CreateLoginProfile"}

    def rule(self, event):
        # Only look for successes
        if event.get("errorCode") or event.get("errorMessage"):
            return False
        # Check when someone other than the user themselves creates or modifies a login profile
        return (
            event.get("eventSource", "") == "iam.amazonaws.com"
            and event.get("eventName", "") in self.PROFILE_EVENTS
            and (
                not deep_get(event, "userIdentity", "arn", default="").endswith(
                    f"/{deep_get(event, 'requestParameters', 'userName', default='')}",
                )
            )
        )

    def title(self, event):
        return f"[{deep_get(event, 'userIdentity', 'arn')}] changed the password for [{deep_get(event, 'requestParameters', 'userName')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_branch_protection_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Branch Protection Disabled",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "protected_branch.destroy",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Protected Branch Name Updated",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "protected_branch.update_name",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GitHubBranchProtectionDisabled(Rule):
    id = "GitHub.Branch.ProtectionDisabled-prototype"
    display_name = "GitHub Branch Protection Disabled"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Initial Access:Supply Chain Compromise"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195"]}
    default_severity = Severity.HIGH
    default_description = "Disabling branch protection controls could indicate malicious use of admin credentials in an attempt to hide activity."
    default_runbook = "Verify that branch protection should be disabled on the repository and re-enable as necessary."
    default_reference = "https://docs.github.com/en/repositories/configuring-branches-and-merges-in-your-repository/managing-protected-branches/managing-a-branch-protection-rule"
    tests = git_hub_branch_protection_disabled_tests

    def rule(self, event):
        return event.get("action") == "protected_branch.destroy"

    def title(self, event):
        return f"A branch protection was removed from the repository [{event.get('repo', '<UNKNOWN_REPO>')}] by [{event.get('actor', '<UNKNOWN_ACTOR>')}]"



from pypanther.base import DataModel as DataModel
from pypanther.base import DataModelMapping as DataModelMapping
from pypanther.base import Rule as Rule
from pypanther.base import panther_managed as panther_managed
from pypanther.get import get_panther_rules as get_panther_rules
from pypanther.get import get_rules as get_rules
from pypanther.log_types import LogType as LogType
from pypanther.registry import register as register
from pypanther.registry import registered_rules as registered_rules
from pypanther.severity import Severity as Severity
from pypanther.unit_tests import RuleMock as RuleMock
from pypanther.unit_tests import RuleTest as RuleTest



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import get_crowdstrike_field

crowdstrike_real_time_response_session_tests: list[RuleTest] = [
    RuleTest(
        name="RTS session start event",
        expected_result=True,
        log={
            "cid": "12345abcdef",
            "unknown_payload": {
                "AgentIdString": "12ab56cd",
                "CustomerIdString": "1234",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionStartEvent",
                "HostnameField": "John Macbook Pro",
                "Nonce": -4714046577736361000,
                "SessionId": "6e1181e4-4924-4761-az3d-666851jdb950",
                "StartTimestamp": 1670460538,
                "UTCTimestamp": 1670460538000,
                "UserName": "example@example.io",
                "cid": "12345abcdef",
                "eid": 118,
                "timestamp": "2022-12-08T00:48:58Z",
            },
        },
    ),
    RuleTest(
        name="RTS session not started",
        expected_result=False,
        log={
            "cid": "12345abcdef",
            "unknown_payload": {
                "AgentIdString": "12ab56cd",
                "CustomerIdString": "1234",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionEndEvent",
                "HostnameField": "John Macbook Pro",
                "Nonce": -4714046577736361000,
                "SessionId": "6e1181e4-4924-4761-az3d-666851jdb950",
                "StartTimestamp": 1670460538,
                "UTCTimestamp": 1670460538000,
                "UserName": "example@example.io",
                "cid": "12345abcdef",
                "eid": 118,
                "timestamp": "2022-12-08T00:48:58Z",
            },
        },
    ),
    RuleTest(
        name="RTS session start event (FDREvent)",
        expected_result=True,
        log={
            "event": {
                "AgentIdString": "42db160eec7948658374a28a4088f297",
                "CustomerIdString": "712bcd164963442ea43d52917cecdecc",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionStartEvent",
                "HostnameField": "US-C02TEST",
                "Nonce": "13732697495973190000",
                "SessionId": "6e1081e4-4914-4761-af3d-666851adb950",
                "StartTimestamp": "1670460538",
                "UTCTimestamp": "1670460538000",
                "UserName": "someone@runpanther.io",
                "cid": "",
                "eid": "118",
                "timestamp": "2022-12-08T00:48:58Z",
            },
            "timestamp": "2022-12-08 00:48:58.000000000",
            "aid": "42db160eec7948658374a28a4088f297",
            "cid": "712bcd164963442ea43d52917cecdecc",
            "fdr_event_type": "Event_RemoteResponseSessionStartEvent",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_event_time": "2022-12-08T00:48:58Z",
            "p_any_domain_names": ["US-C02TEST"],
            "p_any_md5_hashes": ["42db160eec7948658374a28a4088f297", "712bcd164963442ea43d52917cecdecc"],
            "p_any_trace_ids": ["42db160eec7948658374a28a4088f297", "712bcd164963442ea43d52917cecdecc"],
            "p_any_usernames": ["someone@runpanther.io"],
            "p_any_emails": ["someone@runpanther.io"],
        },
    ),
    RuleTest(
        name="RTS session not started (FDREvent)",
        expected_result=False,
        log={
            "event": {
                "AgentIdString": "42db160eec7948658374a28a4088f297",
                "CustomerIdString": "712bcd164963442ea43d52917cecdecc",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionEndEvent",
                "HostnameField": "US-C02TEST",
                "Nonce": "13732697495973190000",
                "SessionId": "6e1081e4-4914-4761-af3d-666851adb950",
                "StartTimestamp": "1670460538",
                "UTCTimestamp": "1670460538000",
                "UserName": "someone@runpanther.io",
                "cid": "",
                "eid": "118",
                "timestamp": "2022-12-08T00:48:58Z",
            },
            "timestamp": "2022-12-08 00:48:58.000000000",
            "aid": "42db160eec7948658374a28a4088f297",
            "cid": "712bcd164963442ea43d52917cecdecc",
            "fdr_event_type": "Event_RemoteResponseSessionEndEvent",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_event_time": "2022-12-08T00:48:58Z",
            "p_any_domain_names": ["US-C02TEST"],
            "p_any_md5_hashes": ["42db160eec7948658374a28a4088f297", "712bcd164963442ea43d52917cecdecc"],
            "p_any_trace_ids": ["42db160eec7948658374a28a4088f297", "712bcd164963442ea43d52917cecdecc"],
            "p_any_usernames": ["someone@runpanther.io"],
            "p_any_emails": ["someone@runpanther.io"],
        },
    ),
]


@panther_managed
class CrowdstrikeRealTimeResponseSession(Rule):
    display_name = "Crowdstrike Real Time Response (RTS) Session"
    id = "Crowdstrike.RealTimeResponse.Session-prototype"
    default_severity = Severity.MEDIUM
    log_types = [LogType.CROWDSTRIKE_UNKNOWN, LogType.CROWDSTRIKE_FDR_EVENT]
    tags = ["Crowdstrike"]
    default_description = "Alert when someone uses Crowdstrike’s RTR (real-time response) capability to access a machine remotely to run commands.\n"
    default_runbook = "Validate the real-time response session started by the Actor.\n"
    default_reference = "https://falcon.us-2.crowdstrike.com/documentation/71/real-time-response-and-network-containment#reviewing-real-time-response-audit-logs"
    tests = crowdstrike_real_time_response_session_tests

    def rule(self, event):
        return (
            get_crowdstrike_field(event, "ExternalApiType", default="<unknown-ExternalApiType>")
            == "Event_RemoteResponseSessionStartEvent"
        )

    def title(self, event):
        user_name = get_crowdstrike_field(event, "UserName", default="<unknown-UserName>")
        hostname_field = get_crowdstrike_field(event, "HostnameField", default="<unknown-HostNameField>")
        return f"{user_name} started a Crowdstrike Real-Time Response (RTR) shell on {hostname_field}"

    def alert_context(self, event):
        return {
            "Start Time": get_crowdstrike_field(event, "StartTimestamp", default="<unknown-StartTimestamp>"),
            "SessionId": get_crowdstrike_field(event, "SessionId", default="<unknown-SessionId>"),
            "Actor": get_crowdstrike_field(event, "UserName", default="<unknown-UserName>"),
            "Target Host": get_crowdstrike_field(event, "HostnameField", default="<unknown-HostnameField>"),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_duo_helpers import deserialize_administrator_log_event_description, duo_alert_context

duo_admin_ssosaml_requirement_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Enforcement Disabled",
        expected_result=True,
        log={
            "action": "admin_single_sign_on_update",
            "description": '{"enforcement_status": "disabled"}',
            "isotimestamp": "2021-10-12 21:29:22",
            "timestamp": "2021-10-12 21:29:22",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Enforcement Optional",
        expected_result=True,
        log={
            "action": "admin_single_sign_on_update",
            "description": '{"enforcement_status": "optional"}',
            "isotimestamp": "2021-10-12 21:29:22",
            "timestamp": "2021-10-12 21:29:22",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Enforcement Required",
        expected_result=False,
        log={
            "action": "admin_single_sign_on_update",
            "description": '{"enforcement_status": "required"}',
            "isotimestamp": "2021-10-12 21:29:22",
            "timestamp": "2021-10-12 21:29:22",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="SSO Update",
        expected_result=False,
        log={
            "action": "admin_single_sign_on_update",
            "description": '{"sso_url": "https://duff.okta.com/app/duoadminpanel/abcdefghijklm/sso/saml", "slo_url": null, "idp_type": "okta", "cert": "C=US/CN=duff/L=Springfield/O=Okta/OU=SSOProvider/ST=California/emailAddress=info@okta.com - 2031-08-10 13:39:00+00:00", "require_signed_response": true, "entity_id": "http://www.okta.com/abcdefghijk"}',
            "isotimestamp": "2021-10-12 21:33:40",
            "timestamp": "2021-10-12 21:33:40",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminSSOSAMLRequirementDisabled(Rule):
    default_description = "Detects when SAML Authentication for Administrators is marked as Disabled or Optional."
    display_name = "Duo Admin SSO SAML Requirement Disabled"
    default_reference = "https://duo.com/docs/sso#saml:~:text=Modify%20Authentication%20Sources"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.SSO.SAML.Requirement.Disabled-prototype"
    tests = duo_admin_ssosaml_requirement_disabled_tests

    def rule(self, event):
        if event.get("action") == "admin_single_sign_on_update":
            description = deserialize_administrator_log_event_description(event)
            enforcement_status = description.get("enforcement_status", "required")
            return enforcement_status != "required"
        return False

    def title(self, event):
        description = deserialize_administrator_log_event_description(event)
        return f"Duo: [{event.get('username', '<username_not_found>')}] changed SAML authentication requirements for Administrators to [{description.get('enforcement_status', '<enforcement_status_not_found>')}]"

    def alert_context(self, event):
        return duo_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

duo_admin_bypass_code_created_tests: list[RuleTest] = [
    RuleTest(
        name="Bypass Create",
        expected_result=True,
        log={
            "action": "bypass_create",
            "description": '{"bypass": "", "count": 1, "valid_secs": 3600, "auto_generated": true, "remaining_uses": 1, "user_id": "D12345", "bypass_code_ids": ["A12345"]}',
            "isotimestamp": "2022-12-14 21:17:39",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:39",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Bypass Delete",
        expected_result=False,
        log={
            "action": "bypass_detele",
            "description": '{"bypass": "", "count": 1, "valid_secs": 3600, "auto_generated": true, "remaining_uses": 1, "user_id": "D12345", "bypass_code_ids": ["A12345"]}',
            "isotimestamp": "2022-12-14 21:17:39",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:39",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminBypassCodeCreated(Rule):
    default_description = "A Duo administrator created an MFA bypass code for an application."
    display_name = "Duo Admin Bypass Code Created"
    default_runbook = "Confirm this was authorized and necessary behavior."
    default_reference = "https://duo.com/docs/administration-users#generating-a-bypass-code"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.Bypass.Code.Created-prototype"
    tests = duo_admin_bypass_code_created_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return event.get("action", "") == "bypass_create"

    def title(self, event):
        # If no 'dedup' function is defined, the return value of
        # this method will act as deduplication string.
        return f"Duo: [{event.get('username', '<NO_USER_FOUND>')}] created a MFA bypass code for [{event.get('object', '<NO_OBJECT_FOUND>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

google_workspace_apps_marketplace_allowlist_tests: list[RuleTest] = [
    RuleTest(
        name="parameters json key set to null value",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "111111111111111111111"},
            "id": {
                "applicationName": "user_accounts",
                "customerId": "C00000000",
                "time": "2022-12-29 22:42:44.467000000",
                "uniqueQualifier": "517500000000000000",
            },
            "parameters": None,
            "ipAddress": "2600:2600:2600:2600:2600:2600:2600:2600",
            "kind": "admin#reports#activity",
            "name": "recovery_email_edit",
            "type": "recovery_info_change",
        },
    ),
    RuleTest(
        name="Change Email Setting",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-10 23:38:45.125000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_EMAIL_SETTING",
            "parameters": {
                "NEW_VALUE": "3",
                "OLD_VALUE": "2",
                "ORG_UNIT_NAME": "EXAMPLE IO",
                "SETTING_NAME": "ENABLE_G_SUITE_MARKETPLACE",
            },
            "type": "EMAIL_SETTINGS",
        },
    ),
    RuleTest(
        name="Change Email Setting Default",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D1234",
                "time": "2022-12-10 23:33:04.667000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_EMAIL_SETTING",
            "parameters": {
                "NEW_VALUE": "1",
                "OLD_VALUE": "DEFAULT",
                "ORG_UNIT_NAME": "EXAMPLE IO",
                "SETTING_NAME": "ENABLE_G_SUITE_MARKETPLACE",
            },
            "type": "EMAIL_SETTINGS",
        },
    ),
    RuleTest(
        name="New Custom Role Created",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "123456"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 02:57:48.693000000",
                "uniqueQualifier": "-12456",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CREATE_ROLE",
            "parameters": {"ROLE_ID": "567890", "ROLE_NAME": "CustomAdminRoleName"},
            "type": "DELEGATED_ADMIN_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GoogleWorkspaceAppsMarketplaceAllowlist(Rule):
    default_description = "Google Workspace Marketplace application allowlist settings were modified."
    display_name = "Google Workspace Apps Marketplace Allowlist"
    default_runbook = "Confirm with the acting user that this change was authorized."
    default_reference = "https://support.google.com/a/answer/6089179?hl=en"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    id = "Google.Workspace.Apps.Marketplace.Allowlist-prototype"
    tests = google_workspace_apps_marketplace_allowlist_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        setting_name = deep_get(event, "parameters", "SETTING_NAME", default="<NO_SETTING_NAME>")
        old_val = deep_get(event, "parameters", "OLD_VALUE", default="<NO_OLD_VALUE_FOUND>")
        new_val = deep_get(event, "parameters", "NEW_VALUE", default="<NO_NEW_VALUE_FOUND>")
        return setting_name == "ENABLE_G_SUITE_MARKETPLACE" and old_val != new_val

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this
        # method will act as deduplication string.
        value_dict = {
            "DEFAULT": "DEFAULT",
            "1": "Don't allow users to install and run apps from the Marketplace",
            "2": "Allow users to install and run any app from the Marketplace",
            "3": "Allow users to install and run only selected apps from the Marketplace",
        }
        old_val = deep_get(event, "parameters", "OLD_VALUE", default="<NO_OLD_VALUE_FOUND>")
        new_val = deep_get(event, "parameters", "NEW_VALUE", default="<NO_NEW_VALUE_FOUND>")
        actor = deep_get(event, "actor", "email", default="<NO_EMAIL_FOUND>")
        return f"Google Workspace User [{actor}] made an application allowlist setting change from [{value_dict.get(str(old_val))}] to [{value_dict.get(str(new_val))}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcp_destructive_queries_tests: list[RuleTest] = [
    RuleTest(
        name="Drop Table Event",
        expected_result=True,
        log={
            "insertid": "abcdefghijklmn",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Fdata_access",
            "operation": {
                "id": "1234567890123-gcp-project1:abcdefghijklmnopqrstuvwz",
                "last": True,
                "producer": "bigquery.googleapis.com",
            },
            "p_any_emails": ["user@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-28 18:37:06.079",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-28 18:38:14.478",
            "p_row_id": "06bf03d9d5dfbadba981899e1787bf05",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "gcp-logsource",
            "protopayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@company.io"},
                "authorizationInfo": [
                    {"granted": True, "permission": "bigquery.jobs.create", "resource": "projects/gcp-project1"},
                ],
                "metadata": {
                    "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                    "jobChange": {
                        "after": "DONE",
                        "job": {
                            "jobConfig": {
                                "queryConfig": {
                                    "createDisposition": "CREATE_IF_NEEDED",
                                    "destinationTable": "projects/gcp-project1/datasets/test1/tables/newtable",
                                    "priority": "QUERY_INTERACTIVE",
                                    "query": "DROP TABLE test1.newtable",
                                    "statementType": "DROP_TABLE",
                                    "writeDisposition": "WRITE_EMPTY",
                                },
                                "type": "QUERY",
                            },
                            "jobName": "projects/gcp-project1/jobs/abcdefghijklmnopqrstuvwz",
                            "jobStats": {
                                "createTime": "2023-03-28T18:37:05.842Z",
                                "endTime": "2023-03-28T18:37:06.073Z",
                                "queryStats": {},
                                "startTime": "2023-03-28T18:37:05.934Z",
                            },
                            "jobStatus": {"jobState": "DONE"},
                        },
                    },
                },
                "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                },
                "resourceName": "projects/gcp-project1/jobs/abcdefghijklmnopqrstuvwz",
                "serviceName": "bigquery.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-03-28 18:37:06.745",
            "resource": {"labels": {"location": "US", "project_id": "gcp-project1"}, "type": "bigquery_project"},
            "severity": "INFO",
            "timestamp": "2023-03-28 18:37:06.079",
        },
    ),
    RuleTest(
        name="TableDeletion",
        expected_result=True,
        log={
            "insertid": "abcdefghijklmn",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "1234567890123-gcp-project1:abcdefghijklmnopqrstuvwz",
                "last": True,
                "producer": "bigquery.googleapis.com",
            },
            "p_any_emails": ["user@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-28 18:37:06.079",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-28 18:38:14.478",
            "p_row_id": "06bf03d9d5dfbadba981899e1787bf05",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "gcp-logsource",
            "protopayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "bigquery.tables.delete",
                        "resource": "projects/gcp-project1/datasets/test1/tables/newtable",
                    },
                ],
                "metadata": {
                    "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                    "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    },
                    "resourceName": "projects/gcp-project1/datasets/test1/tables/newtable",
                    "serviceName": "bigquery.googleapis.com",
                    "status": {},
                    "tableDeletion": {
                        "jobName": "projects/gcp-project1/jobs/bquxjob_5e4a0679_18729a639d7",
                        "reason": "QUERY",
                    },
                },
                "receivetimestamp": "2023-03-28 18:37:06.745",
                "resource": {
                    "labels": {"dataset_id": "test1", "project_id": "gcp-project1"},
                    "type": "bigquery_dataset",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-28 18:37:06.079",
            },
        },
    ),
]


@panther_managed
class GCPDestructiveQueries(Rule):
    default_description = (
        "Detect any destructive BigQuery queries or jobs such as update, delete, drop, alter or truncate."
    )
    display_name = "GCP Destructive Queries"
    default_reference = "https://cloud.google.com/bigquery/docs/managing-tables"
    default_severity = Severity.INFO
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Destructive.Queries-prototype"
    tests = gcp_destructive_queries_tests
    DESTRUCTIVE_STATEMENTS = ["UPDATE", "DELETE", "DROP_TABLE", "ALTER_TABLE", "TRUNCATE_TABLE"]

    def rule(self, event):
        if all(
            [
                deep_get(event, "resource", "type", default="<RESOURCE_NOT_FOUND>").startswith("bigquery"),
                deep_get(event, "protoPayload", "metadata", "jobChange", "job", "jobConfig", "type") == "QUERY",
                deep_get(
                    event,
                    "protoPayload",
                    "metadata",
                    "jobChange",
                    "job",
                    "jobConfig",
                    "queryConfig",
                    "statementType",
                    default="<STATEMENT_NOT_FOUND>",
                )
                in self.DESTRUCTIVE_STATEMENTS,
            ],
        ):
            return True
        if deep_get(event, "protoPayload", "metadata", "tableDeletion"):
            return True
        if deep_get(event, "protoPayload", "metadata", "datasetDeletion"):
            return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        statement = deep_get(
            event,
            "protoPayload",
            "metadata",
            "jobChange",
            "job",
            "jobConfig",
            "queryConfig",
            "statementType",
            default="<STATEMENT_NOT_FOUND>",
        )
        table = deep_get(
            event,
            "protoPayload",
            "metadata",
            "jobChange",
            "job",
            "jobConfig",
            "queryConfig",
            "destinationTable",
        ) or deep_get(event, "protoPayload", "metadata", "resourceName", default="<TABLE_NOT_FOUND>")
        return f"GCP: [{actor}] performed a destructive BigQuery [{statement}] query on [{table}]."

    def alert_context(self, event):
        return {
            "query": deep_get(
                event,
                "protoPayload",
                "metadata",
                "jobChange",
                "job",
                "jobConfig",
                "queryConfig",
                "query",
                default="<QUERY_NOT_FOUND>",
            ),
            "actor": deep_get(
                event,
                "protoPayload",
                "authenticationInfo",
                "principalEmail",
                default="<ACTOR_NOT_FOUND>",
            ),
            "statement": deep_get(
                event,
                "protoPayload",
                "metadata",
                "jobChange",
                "job",
                "jobConfig",
                "queryConfig",
                "statementType",
                default="<STATEMENT_NOT_FOUND>",
            ),
            "table": deep_get(
                event,
                "protoPayload",
                "metadata",
                "jobChange",
                "job",
                "jobConfig",
                "queryConfig",
                "destinationTable",
            )
            or deep_get(event, "protoPayload", "metadata", "resourceName", default="<TABLE_NOT_FOUND>"),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcp_big_query_large_scan_tests: list[RuleTest] = [
    RuleTest(
        name="small query",
        expected_result=False,
        log={
            "insertid": "ABCDEFGHIJKL",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Fdata_access",
            "operation": {
                "id": "0123456789012-gcp-project1:abcdefg_hijklmnop_1234567abcd",
                "last": True,
                "producer": "bigquery.googleapis.com",
            },
            "p_any_emails": ["user@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-28 17:37:02.096",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-28 17:38:14.621",
            "p_row_id": "de00d3dcdaeee4b4d5f7fa9d17b4c203",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "gcp-logsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@company.io"},
                "authorizationInfo": [
                    {"granted": True, "permission": "bigquery.jobs.create", "resource": "projects/gcp-project1"},
                ],
                "metadata": {
                    "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                    "jobChange": {
                        "after": "DONE",
                        "job": {
                            "jobConfig": {
                                "queryConfig": {
                                    "createDisposition": "CREATE_IF_NEEDED",
                                    "destinationTable": "projects/gcp-project1/datasets/_c2b49f742788f188022fcec1f1622e7404b40ce5/tables/anondea8e77183dcdf1cf93a47b5003036409a525808207450ab80db17fcc8cdcf53",
                                    "priority": "QUERY_INTERACTIVE",
                                    "query": "-- This query shows a list of the daily top Google Search terms.\nSELECT\n   *\nFROM `bigquery-public-data.google_trends.top_terms`",
                                    "statementType": "SELECT",
                                    "writeDisposition": "WRITE_TRUNCATE",
                                },
                                "type": "QUERY",
                            },
                            "jobName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                            "jobStats": {
                                "createTime": "2023-03-28T17:36:49.087Z",
                                "endTime": "2023-03-28T17:37:02.092Z",
                                "queryStats": {
                                    "billingTier": 1,
                                    "outputRowCount": "43683701",
                                    "referencedTables": [
                                        "projects/bigquery-public-data/datasets/google_trends/tables/top_terms",
                                    ],
                                    "totalBilledBytes": "100000",
                                    "totalProcessedBytes": "100000",
                                },
                                "startTime": "2023-03-28T17:36:49.224Z",
                                "totalSlotMs": "259581",
                            },
                            "jobStatus": {"jobState": "DONE"},
                        },
                    },
                },
                "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                },
                "resourceName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                "serviceName": "bigquery.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-03-28 17:37:02.114",
            "resource": {"labels": {"location": "US", "project_id": "gcp-project1"}, "type": "bigquery_project"},
            "severity": "INFO",
            "timestamp": "2023-03-28 17:37:02.096",
        },
    ),
    RuleTest(
        name="Large Query",
        expected_result=True,
        log={
            "insertid": "ABCDEFGHIJKL",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Fdata_access",
            "operation": {
                "id": "0123456789012-gcp-project1:abcdefg_hijklmnop_1234567abcd",
                "last": True,
                "producer": "bigquery.googleapis.com",
            },
            "p_any_emails": ["user@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-28 17:37:02.096",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-28 17:38:14.621",
            "p_row_id": "de00d3dcdaeee4b4d5f7fa9d17b4c203",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "gcp-logsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@company.io"},
                "authorizationInfo": [
                    {"granted": True, "permission": "bigquery.jobs.create", "resource": "projects/gcp-project1"},
                ],
                "metadata": {
                    "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                    "jobChange": {
                        "after": "DONE",
                        "job": {
                            "jobConfig": {
                                "queryConfig": {
                                    "createDisposition": "CREATE_IF_NEEDED",
                                    "destinationTable": "projects/gcp-project1/datasets/_c2b49f742788f188022fcec1f1622e7404b40ce5/tables/anondea8e77183dcdf1cf93a47b5003036409a525808207450ab80db17fcc8cdcf53",
                                    "priority": "QUERY_INTERACTIVE",
                                    "query": "-- This query shows a list of the daily top Google Search terms.\nSELECT\n   *\nFROM `bigquery-public-data.google_trends.top_terms`",
                                    "statementType": "SELECT",
                                    "writeDisposition": "WRITE_TRUNCATE",
                                },
                                "type": "QUERY",
                            },
                            "jobName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                            "jobStats": {
                                "createTime": "2023-03-28T17:36:49.087Z",
                                "endTime": "2023-03-28T17:37:02.092Z",
                                "queryStats": {
                                    "billingTier": 1,
                                    "outputRowCount": "43683701",
                                    "referencedTables": [
                                        "projects/bigquery-public-data/datasets/google_trends/tables/top_terms",
                                    ],
                                    "totalBilledBytes": "3097493504",
                                    "totalProcessedBytes": "3096864198",
                                },
                                "startTime": "2023-03-28T17:36:49.224Z",
                                "totalSlotMs": "259581",
                            },
                            "jobStatus": {"jobState": "DONE"},
                        },
                    },
                },
                "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                },
                "resourceName": "projects/gcp-project1/jobs/abcdefg_hijklmnop_1234567abcd",
                "serviceName": "bigquery.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-03-28 17:37:02.114",
            "resource": {"labels": {"location": "US", "project_id": "gcp-project1"}, "type": "bigquery_project"},
            "severity": "INFO",
            "timestamp": "2023-03-28 17:37:02.096",
        },
    ),
]


@panther_managed
class GCPBigQueryLargeScan(Rule):
    default_description = "Detect any BigQuery query that is doing a very large scan (> 1 GB)."
    display_name = "GCP BigQuery Large Scan"
    default_reference = "https://cloud.google.com/bigquery/docs/running-queries"
    default_severity = Severity.INFO
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.BigQuery.Large.Scan-prototype"
    tests = gcp_big_query_large_scan_tests
    # 1.07 GB
    QUERY_THRESHOLD_BYTES = 1073741824

    def rule(self, event):
        return all(
            [
                deep_get(event, "resource", "type", default="<type not found>").startswith("bigquery"),
                deep_get(event, "operation", "last") is True,
                deep_get(event, "protoPayload", "metadata", "jobChange", "job", "jobConfig", "type") == "QUERY",
                deep_get(
                    event,
                    "protoPayload",
                    "metadata",
                    "jobChange",
                    "job",
                    "jobConfig",
                    "queryConfig",
                    "statementType",
                )
                == "SELECT",
                int(
                    deep_get(
                        event,
                        "protoPayload",
                        "metadata",
                        "jobChange",
                        "job",
                        "jobStats",
                        "queryStats",
                        "totalBilledBytes",
                        default=0,
                    ),
                )
                > self.QUERY_THRESHOLD_BYTES,
            ],
        )

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        query_size = deep_get(
            event,
            "protoPayload",
            "metadata",
            "jobChange",
            "job",
            "jobStats",
            "queryStats",
            "totalBilledBytes",
            default=0,
        )
        return f"GCP: [{actor}] ran a large BigQuery query of [{query_size}] bytes."

    def alert_context(self, event):
        return {
            "query": deep_get(
                event,
                "protoPayload",
                "metadata",
                "jobChange",
                "job",
                "jobConfig",
                "queryConfig",
                "query",
                default="<QUERY_NOT_FOUND>",
            ),
            "actor": deep_get(
                event,
                "protoPayload",
                "authenticationInfo",
                "principalEmail",
                default="<ACTOR_NOT_FOUND>",
            ),
            "query_size": deep_get(
                event,
                "protoPayload",
                "metadata",
                "jobChange",
                "job",
                "jobStats",
                "queryStats",
                "totalBilledBytes",
                default=0,
            ),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

carbon_black_audit_flagged_tests: list[RuleTest] = [
    RuleTest(
        name="Flagged",
        expected_result=True,
        log={
            "clientIp": "12.34.56.78",
            "description": "User bob.ross@acme.com retrieved secret for API ID JFDNIPS464 in org 12345",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": True,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Other role granted",
        expected_result=False,
        log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Read Only",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Sensor update requested",
        expected_result=True,
        log={
            "description": "Requested sensor update to version: 2.16.0.2566828 for the following device: ABCDEFG012 (ID: 21360056)",
            "eventId": "ac5f46923e9c11efaadd07ba65d6cd7b",
            "eventTime": "2024-07-10 09:13:29.952000000",
            "flagged": True,
            "loginName": "",
            "orgName": "acme.com",
            "requestUrl": "/settings/users/pushSensorKits",
            "verbose": False,
        },
    ),
]


@panther_managed
class CarbonBlackAuditFlagged(Rule):
    id = "CarbonBlack.Audit.Flagged-prototype"
    log_types = [LogType.CARBON_BLACK_AUDIT]
    default_description = (
        "Detects when Carbon Black has flagged a log as important, such as failed login attempts and locked accounts."
    )
    display_name = "Carbon Black Log Entry Flagged"
    default_severity = Severity.MEDIUM
    tags = ["Credential Access", "Brute Force"]
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-FB61E4E3-6431-4226-A4E3-5949FB75922B.html"
    tests = carbon_black_audit_flagged_tests

    def rule(self, event):
        return event.get("flagged", False)

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        desc = event.get("description", "<NO_DESCRIPTION_FOUND>")
        return f"{user} [{ip_addr}] {desc}"

    def severity(self, event):
        if event.get("description").startswith("Requested sensor update"):
            return "INFO"
        return "DEFAULT"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_system_sso_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk System SSO Setting event happened",
        expected_result=True,
        log={
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
            "event": "group.sso.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "content": {"unknown": "contents"},
        },
    ),
    RuleTest(
        name="Snyk Group SSO Membership sync",
        expected_result=False,
        log={
            "content": {
                "addAsOrgAdmin": [],
                "addAsOrgCollaborator": ["group.name"],
                "addAsOrgCustomRole": [],
                "addAsOrgRestrictedCollaborator": [],
                "removedOrgMemberships": [],
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


@panther_managed
class SnykSystemSSO(Rule):
    id = "Snyk.System.SSO-prototype"
    display_name = "Snyk System SSO Settings Changed"
    log_types = [LogType.SNYK_GROUP_AUDIT]
    tags = ["Snyk"]
    default_severity = Severity.HIGH
    default_description = "Detects Snyk SSO Settings have been changed. The reference URL from Snyk indicates that these events are likely to originate exclusively from Snyk Support.\n"
    default_reference = "https://docs.snyk.io/user-and-group-management/setting-up-sso-for-authentication/set-up-snyk-single-sign-on-sso"
    summary_attributes = ["event", "p_any_ip_addresses", "p_any_emails"]
    tests = snyk_system_sso_tests
    ACTIONS = [
        "group.sso.auth0_connection.create",
        "group.sso.auth0_connection.edit",
        "group.sso.create",
        "group.sso.edit",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        return f"Snyk: System SSO Setting event [{deep_get(event, 'event', default='<NO_EVENT>')}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        return snyk_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_unsuccessful_mf_aattempt_tests: list[RuleTest] = [
    RuleTest(
        name="Successful Login w/ MFA",
        expected_result=False,
        log={
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/home?state=hashArgs%23&isauthcode=true",
                "MFAIdentifier": "arn:aws:iam::111122223333:u2f/user/anaya/default-AAAAAAAABBBBBBBBCCCCCCCCDD",
                "MFAUsed": "Yes",
                "MobileVersion": "No",
            },
            "awsRegion": "us-east-1",
            "eventID": "fed06f42-cb12-4764-8c69-121063dc79b9",
            "eventName": "ConsoleLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2022-11-10T16:24:34Z",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.05",
            "recipientAccountId": "111122223333",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36",
            "userIdentity": {
                "accountId": "111122223333",
                "arn": "arn:aws:iam::111122223333:user/anaya",
                "principalId": "AIDACKCEVSQ6C2EXAMPLE",
                "type": "IAMUser",
                "userName": "anaya",
            },
        },
    ),
    RuleTest(
        name="Unsuccessful Login w/ MFA",
        expected_result=True,
        log={
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/home?state=hashArgs%23&isauthcode=true",
                "MFAUsed": "Yes",
                "MobileVersion": "No",
            },
            "awsRegion": "us-east-1",
            "errorMessage": "Failed authentication",
            "eventID": "d38ce1b3-4575-4cb8-a632-611b8243bfc3",
            "eventName": "ConsoleLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2022-11-10T16:24:34Z",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.05",
            "recipientAccountId": "111122223333",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Failure"},
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "",
                "accountId": "111122223333",
                "principalId": "AIDACKCEVSQ6C2EXAMPLE",
                "type": "IAMUser",
                "userName": "anaya",
            },
        },
    ),
]


@panther_managed
class AWSUnsuccessfulMFAattempt(Rule):
    default_description = "Monitor application logs for suspicious events including repeated MFA failures that may indicate user's primary credentials have been compromised."
    display_name = "AWS Unsuccessful MFA attempt"
    enabled = False
    default_reference = "https://attack.mitre.org/techniques/T1621/"
    tags = ["Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0006:T1621"]}
    default_severity = Severity.HIGH
    dedup_period_minutes = 15
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.Unsuccessful.MFA.attempt-prototype"
    threshold = 2
    tests = aws_unsuccessful_mf_aattempt_tests

    def rule(self, event):
        if event.get("eventSource") != "signin.amazonaws.com" and event.get("eventName") != "ConsoleLogin":
            return False
        mfa_used = deep_get(event, "additionalEventData", "MFAUsed", default="")
        console_login = deep_get(event, "responseElements", "ConsoleLogin", default="")
        if mfa_used == "Yes" and console_login == "Failure":
            return True
        return False

    def title(self, event):
        arn = deep_get(event, "userIdenity", "arn", default="No ARN")
        username = deep_get(event, "userIdentity", "userName", default="No Username")
        return f"Failed MFA login from [{arn}] [{username}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_leaked_password_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "type": "login",
            "name": "logout",
            "parameters": {"login_type": "saml"},
        },
    ),
    RuleTest(
        name="Account Warning Not For Password Leaked",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "type": "account_warning",
            "name": "account_disabled_spamming",
            "parameters": {"affected_email_address": "homer.simpson@example.com"},
        },
    ),
    RuleTest(
        name="Account Warning For Password Leaked",
        expected_result=True,
        log={
            "id": {"applicationName": "login"},
            "type": "account_warning",
            "name": "account_disabled_password_leak",
            "parameters": {"affected_email_address": "homer.simpson@example.com"},
        },
    ),
]


@panther_managed
class GSuiteLeakedPassword(Rule):
    id = "GSuite.LeakedPassword-prototype"
    display_name = "GSuite User Password Leaked"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Credential Access:Unsecured Credentials"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.HIGH
    default_description = "GSuite reported a user's password has been compromised, so they disabled the account.\n"
    default_reference = "https://support.google.com/a/answer/2984349?hl=en#zippy=%2Cstep-temporarily-suspend-the-suspected-compromised-user-account%2Cstep-investigate-the-account-for-unauthorized-activity%2Cstep-revoke-access-to-the-affected-account%2Cstep-return-access-to-the-user-again%2Cstep-enroll-in--step-verification-with-security-keys%2Cstep-add-secure-or-update-recovery-options%2Cstep-enable-account-activity-alerts"
    default_runbook = "GSuite has already disabled the compromised user's account. Consider investigating how the user's account was compromised, and reset their account and password. Advise the user to change any other passwords in use that are the sae as the compromised password.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_leaked_password_tests
    PASSWORD_LEAKED_EVENTS = {"account_disabled_password_leak"}

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "login":
            return False
        if event.get("type") == "account_warning":
            return bool(event.get("name") in self.PASSWORD_LEAKED_EVENTS)
        return False

    def title(self, event):
        user = deep_get(event, "parameters", "affected_email_address")
        if not user:
            user = "<UNKNOWN_USER>"
        return f"User [{user}]'s account was disabled due to a password leak"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_box_helpers import is_box_sdk_enabled, lookup_box_file, lookup_box_folder

box_item_shared_externally_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": 12345678, "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "DELETE",
            "source": {
                "item_name": "regular_file.pdf",
                "item_type": "file",
                "owned_by": {"id": 12345678, "type": "user", "login": "cat@example", "name": "Bob Cat"},
                "parent": {"id": 12345, "type": "folder", "etag": 1, "name": "Parent_Folder", "sequence_id": 2},
            },
        },
    ),
]


@panther_managed
class BoxItemSharedExternally(Rule):
    id = "Box.Item.Shared.Externally-prototype"
    display_name = "Box item shared externally"
    enabled = False
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Exfiltration:Exfiltration Over Web Service", "Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    default_severity = Severity.MEDIUM
    default_description = "A user has shared an item and it is accessible to anyone with the share link (internal or external to the company). This rule requires that the boxsdk[jwt] be installed in the environment.\n"
    default_reference = (
        "https://support.box.com/hc/en-us/articles/4404822772755-Enterprise-Settings-Content-Sharing-Tab"
    )
    default_runbook = "Investigate whether this user's activity is expected.\n"
    summary_attributes = ["ip_address"]
    threshold = 10
    tests = box_item_shared_externally_tests
    ALLOWED_SHARED_ACCESS = {"collaborators", "company"}
    SHARE_EVENTS = {"CHANGE_FOLDER_PERMISSION", "ITEM_SHARED", "ITEM_SHARED_CREATE", "ITEM_SHARED_UPDATE", "SHARE"}

    def rule(self, event):
        # filter events
        if event.get("event_type") not in self.SHARE_EVENTS:
            return False
        # only try to lookup file/folder info if sdk is enabled in the env
        if is_box_sdk_enabled():
            item = self.get_item(event)
            if item is not None and item.get("shared_link"):
                return deep_get(item, "shared_link", "effective_access") not in self.ALLOWED_SHARED_ACCESS
        return False

    def get_item(self, event):
        item_id = deep_get(event, "source", "item_id", default="")
        user_id = deep_get(event, "source", "owned_by", "id", default="")
        item = {}
        if deep_get(event, "source", "item_type") == "folder":
            item = lookup_box_folder(user_id, item_id)
        elif deep_get(event, "source", "item_type") == "file":
            item = lookup_box_file(user_id, item_id)
        return item

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'login', default='<UNKNOWN_USER>')}] shared an item [{deep_get(event, 'source', 'item_name', default='<UNKNOWN_NAME>')}] externally."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_new_saa_s_account_created_tests: list[RuleTest] = [
    RuleTest(
        name="Account Update",
        expected_result=False,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "ATLASSIAN",
                "creationTimestamp": 1698064423.0,
                "email": "john.hill@example.com",
                "employeeId": "72d0347a-2663-4ef5-b1c5-df39163f1603",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "lastUsedTimestamp": 1698669168.0,
                "loginMethods": {
                    "oidcLogin": "GOOGLE_WORKSPACE",
                    "oktaSwaLogin": True,
                    "passwordLogin": True,
                    "samlLogin": "OKTA",
                    "vendorSsoLogin": "GOOGLE_WORKSPACE",
                },
                "mfaMethods": [
                    "APP_TOTP",
                    "PUSH_NOTIFICATION",
                    "EMAIL_OTP",
                    "U2F",
                    "HARDWARE_TOTP",
                    "PHONE_CALL",
                    "SMS_OTP",
                    "APP_PASSWORD",
                    "GRID_CARD",
                    "EXTERNAL_PROVIDER",
                    "BACKUP_CODES",
                    "WEBAUTHN",
                ],
                "mfaRegistered": True,
                "passwordId": "4c13674f-e88a-4411-bfa2-53a70468a898",
            },
            "object": "ACCOUNT",
            "old": {
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "ATLASSIAN",
                "creationTimestamp": 1698064423.0,
                "email": "john.hill@example.com",
                "employeeId": "72d0347a-2663-4ef5-b1c5-df39163f1603",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "lastUsedTimestamp": 1698669168.0,
                "loginMethods": {
                    "oidcLogin": "GOOGLE_WORKSPACE",
                    "oktaSwaLogin": True,
                    "passwordLogin": True,
                    "samlLogin": "OKTA",
                    "vendorSsoLogin": "GOOGLE_WORKSPACE",
                },
                "mfaMethods": [
                    "APP_TOTP",
                    "PUSH_NOTIFICATION",
                    "EMAIL_OTP",
                    "U2F",
                    "HARDWARE_TOTP",
                    "PHONE_CALL",
                    "SMS_OTP",
                    "APP_PASSWORD",
                    "GRID_CARD",
                    "EXTERNAL_PROVIDER",
                    "BACKUP_CODES",
                    "WEBAUTHN",
                ],
                "mfaRegistered": True,
                "passwordId": "4c13674f-e88a-4411-bfa2-53a70468a898",
            },
            "timestamp": 1698604061.0,
            "type": "UPDATE",
            "version": "1",
        },
    ),
    RuleTest(
        name="New Account",
        expected_result=True,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "appType": "ATLASSIAN",
                "creationTimestamp": 1698064423.0,
                "email": "john.hill@example.com",
                "employeeId": "72d0347a-2663-4ef5-b1c5-df39163f1603",
                "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                "lastUsedTimestamp": 1698669168.0,
                "loginMethods": {
                    "oidcLogin": "GOOGLE_WORKSPACE",
                    "oktaSwaLogin": True,
                    "passwordLogin": True,
                    "samlLogin": "OKTA",
                    "vendorSsoLogin": "GOOGLE_WORKSPACE",
                },
                "mfaMethods": [
                    "APP_TOTP",
                    "PUSH_NOTIFICATION",
                    "EMAIL_OTP",
                    "U2F",
                    "HARDWARE_TOTP",
                    "PHONE_CALL",
                    "SMS_OTP",
                    "APP_PASSWORD",
                    "GRID_CARD",
                    "EXTERNAL_PROVIDER",
                    "BACKUP_CODES",
                    "WEBAUTHN",
                ],
                "mfaRegistered": True,
                "passwordId": "4c13674f-e88a-4411-bfa2-53a70468a898",
            },
            "object": "ACCOUNT",
            "old": None,
            "timestamp": 1698604061.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityNewSaaSAccountCreated(Rule):
    id = "Push.Security.New.SaaS.Account.Created-prototype"
    display_name = "Push Security New SaaS Account Created"
    log_types = [LogType.PUSH_SECURITY_ENTITIES]
    default_severity = Severity.INFO
    tests = push_security_new_saa_s_account_created_tests

    def rule(self, event):
        if event.get("object") != "ACCOUNT":
            return False
        if event.get("type") == "CREATE":
            return True
        return False

    def title(self, event):
        app_type = event.deep_get("new", "appType")
        new_email = event.deep_get("new", "email")
        return f"New account on {app_type} created by {new_email}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

google_workspace_admin_custom_role_tests: list[RuleTest] = [
    RuleTest(
        name="Delete Role",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "123456"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 02:57:48.693000000",
                "uniqueQualifier": "-12456",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "DELETE_ROLE",
            "parameters": {"ROLE_ID": "567890", "ROLE_NAME": "CustomAdminRoleName"},
            "type": "DELEGATED_ADMIN_SETTINGS",
        },
    ),
    RuleTest(
        name="New Custom Role Created",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "123456"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 02:57:48.693000000",
                "uniqueQualifier": "-12456",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CREATE_ROLE",
            "parameters": {"ROLE_ID": "567890", "ROLE_NAME": "CustomAdminRoleName"},
            "type": "DELEGATED_ADMIN_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GoogleWorkspaceAdminCustomRole(Rule):
    default_description = "A Google Workspace administrator created a new custom administrator role."
    display_name = "Google Workspace Admin Custom Role"
    default_runbook = "Please review this activity with the administrator and ensure this behavior was authorized."
    default_reference = "https://support.google.com/a/answer/2406043?hl=en#:~:text=under%20the%20limit.-,Create%20a%20custom%20role,-Before%20you%20begin"
    default_severity = Severity.MEDIUM
    tags = ["admin", "administrator", "google workspace", "role"]
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    id = "Google.Workspace.Admin.Custom.Role-prototype"
    summary_attributes = ["name", "type"]
    tests = google_workspace_admin_custom_role_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        # Create Alert if there is a custom role created under delegated admin settings
        return event.get("type", "") == "DELEGATED_ADMIN_SETTINGS" and event.get("name", "") == "CREATE_ROLE"

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method
        # will act as deduplication string.
        return f"Google Workspace Administrator [{event.get('actor', {}).get('email', 'NO_EMAIL_FOUND')}] created a new admin role [{event.get('parameters', {}).get('ROLE_NAME', 'NO_ROLE_NAME_FOUND')}]."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

okta_support_access_tests: list[RuleTest] = [
    RuleTest(
        name="Support Access Granted",
        expected_result=True,
        log={
            "published": "2022-03-22 14:21:53.225",
            "eventType": "user.session.impersonation.grant",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "core.user.impersonation.grant.enabled",
            "displayMessage": "Enable impersonation grant",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                },
                "zone": "null",
            },
            "p_log_type": "Okta.SystemLog",
        },
    ),
    RuleTest(
        name="Login Event",
        expected_result=False,
        log={
            "published": "2022-03-22 14:21:53.225",
            "eventType": "user.session.start",
            "version": "0",
            "severity": "INFO",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                },
                "zone": "null",
            },
            "p_log_type": "Okta.SystemLog",
        },
    ),
]


@panther_managed
class OktaSupportAccess(Rule):
    id = "Okta.Support.Access-prototype"
    display_name = "Okta Support Access Granted"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "DataModel", "Okta", "Initial Access:Trusted Relationship"]
    reports = {"MITRE ATT&CK": ["TA0001:T1199"]}
    default_severity = Severity.MEDIUM
    default_description = "An admin user has granted access to Okta Support to your account"
    default_reference = "https://help.okta.com/en/prod/Content/Topics/Settings/settings-support-access.htm"
    default_runbook = "Contact Admin to ensure this was sanctioned activity"
    dedup_period_minutes = 15
    summary_attributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]
    tests = okta_support_access_tests
    OKTA_SUPPORT_ACCESS_EVENTS = ["user.session.impersonation.grant", "user.session.impersonation.initiate"]

    def rule(self, event):
        return event.get("eventType") in self.OKTA_SUPPORT_ACCESS_EVENTS

    def title(self, event):
        return f"Okta Support Access Granted by {event.udm('actor_user')}"

    def alert_context(self, event):
        context = {"user": event.udm("actor_user"), "ip": event.udm("source_ip"), "event": event.get("eventType")}
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_unused_region_tests: list[RuleTest] = [
    RuleTest(
        name="Authorized region",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "StartQueryExecution",
            "eventSource": "athena.amazonaws.com",
            "eventTime": "2021-10-19 04:50:27",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "clientRequestToken": "1111",
                "queryExecutionContext": {"database": "1111"},
                "queryString": "-- Query 2 different time windows for comparison, return sources that have events over the past time window but not within the current time window.\n-- Past time windows: 2 hours\n-- Current time window: 1 hour\n\nWITH PastCount as\n(\nselect p_source_label, count(*)  as PastTotal\nfrom panther_views.all_logs \nWHERE p_parse_time >= current_timestamp - interval '7200' second\n    and partition_time >= to_unixtime(date_trunc('HOUR', current_timestamp - interval '7200' second))\n    group by p_source_label\n),\nCurrentCount as \n(\nselect  p_source_label, count(*)  as CurrentTotal\nfrom panther_views.all_logs \nWHERE p_parse_time >= current_timestamp - interval '3600' second\n    and partition_time >= to_unixtime(date_trunc('HOUR', current_timestamp - interval '3600' second))\n    group by p_source_label\n)\nSELECT PastCount.*, coalesce(CurrentCount.CurrentTotal, 0) AS CurrentTotal \nFROM CurrentCount \nRIGHT OUTER JOIN PastCount ON PastCount.p_source_label = CurrentCount.p_source_label\nWHERE CurrentTotal IS NULL",
                "resultConfiguration": {},
                "workGroup": "Panther",
            },
            "responseElements": {"queryExecutionId": "fc62d701-d474-46a4-b6cf-de6bafdbdb3c"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "aws-sdk-go/1.38.37 (go1.16; linux; amd64) exec-env/AWS_Lambda_go1.x",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-19T04:01:27Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Unauthorized region, read-only",
        expected_result=False,
        log={
            "awsRegion": "ap-east-1",
            "eventID": "1111",
            "eventName": "Decrypt",
            "eventSource": "kms.amazonaws.com",
            "eventTime": "2021-10-19 00:56:05",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "encryptionAlgorithm": "SYMMETRIC_DEFAULT",
                "encryptionContext": {"aws:s3:arn": "arn:aws:s3:::1111"},
            },
            "resources": [
                {
                    "accountId": "123456789012",
                    "arn": "arn:aws:kms:us-west-2:123456789012:key/1111",
                    "type": "AWS::KMS::Key",
                },
            ],
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-19T00:31:41Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Unauthorized region",
        expected_result=True,
        log={
            "apiVersion": "20140328",
            "awsRegion": "eu-central-1",
            "eventID": "1111",
            "eventName": "CreateLogStream",
            "eventSource": "logs.amazonaws.com",
            "eventTime": "2021-10-21 22:29:06",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "logGroupName": "/aws/lambda/panther-analysis-api",
                "logStreamName": "2021/10/21/[$LATEST]1111",
            },
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "awslambda-worker/1.0 rusoto/0.47.0 rust/1.55.0 linux",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2021-10-21T22:29:02Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSUnusedRegion(Rule):
    id = "AWS.UnusedRegion-prototype"
    display_name = "Unused AWS Region"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Defense Evasion:Unused/Unsupported Cloud Regions", "Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0005:T1535"]}
    default_severity = Severity.HIGH
    default_description = "CloudTrail logged non-read activity from a verboten AWS region."
    default_runbook = (
        "https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_examples_aws-enable-disable-regions.html"
    )
    default_reference = "https://attack.mitre.org/techniques/T1535/"
    summary_attributes = ["eventSource", "eventName", "recipientAccountId", "awsRegion", "p_any_aws_arns"]
    tests = aws_unused_region_tests
    # Define a list of verboten or unused regions
    # Could modify to include expected user mappings: { "123456789012": { "us-west-1", "us-east-2" } }
    UNUSED_REGIONS = {"ap-east-1", "eu-west-3", "eu-central-1"}

    def rule(self, event):
        if event.get("awsRegion", "<UNKNOWN_AWS_REGION>") in self.UNUSED_REGIONS and event.get("readOnly") is False:
            return True
        return False

    def title(self, event):
        aws_username = deep_get(event, "userIdentity", "sessionContext", "sessionIssuer", "userName")
        return f"Non-read-only API call in unused region {event.get('awsRegion', '<UNKNOWN_AWS_REGION>')} by user {aws_username}"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk, okta_alert_context

okta_identity_provider_created_modified_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.authentication.auth_via_mfa",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    RuleTest(
        name="FastPass Phishing Block Event",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "system.idp.lifecycle.create",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaIdentityProviderCreatedModified(Rule):
    id = "Okta.Identity.Provider.Created.Modified-prototype"
    display_name = "Okta Identity Provider Created or Modified"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    reports = {"MITRE ATT&CK": ["TA0006:T1556", "TA0001:T1199", "TA0003:T1098"]}
    default_severity = Severity.HIGH
    default_description = 'A new 3rd party Identity Provider has been created or modified. Attackers have been observed configuring a second Identity Provider to act as an "impersonation app" to access applications within the compromised Org on behalf of other users. This second Identity Provider, also controlled by the attacker, would act as a “source” IdP in an inbound federation relationship (sometimes called “Org2Org”) with the target.\n'
    default_runbook = "Delegate access to this feature to a Custom Admin Role with the minimum required permissions. Constrain these roles to groups that exclude highly privileged administrators.\n"
    default_reference = "https://sec.okta.com/articles/2023/08/cross-tenant-impersonation-prevention-and-detection\n"
    dedup_period_minutes = 30
    tests = okta_identity_provider_created_modified_tests

    def rule(self, event):
        return "system.idp.lifecycle" in event.get("eventType")

    def title(self, event):
        action = event.get("eventType").split(".")[-1]
        target = deep_walk(event, "target", "displayName", default="<displayName-not-found>", return_val="first")
        return f"{deep_get(event, 'actor', 'displayName', default='<displayName-not-found>')} <{deep_get(event, 'actor', 'alternateId', default='alternateId-not-found')}> {action}d Identity Provider [{target}]"

    def severity(self, event):
        if "create" in event.get("eventType"):
            return "HIGH"
        return "MEDIUM"

    def alert_context(self, event):
        return okta_alert_context(event)



from pypanther.base import DataModel, DataModelMapping, LogType

# 1Password item usage logs don't have event types, this file is a placeholder. All events are
# the viewing or usage of an item in 1Password


class StandardOnePasswordItemUsage(DataModel):
    id: str = "Standard.OnePassword.ItemUsage"
    display_name: str = "1Password Item Usage Events"
    enabled: bool = True
    log_types: list[str] = [LogType.ONEPASSWORD_ITEM_USAGE]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.user.email"),
        DataModelMapping(name="source_ip", path="$.client.ipaddress"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_azuresignin_helpers import actor_user, azure_signin_alert_context, is_sign_in_event
from pypanther.helpers.panther_base_helpers import deep_get

azure_audit_many_failed_sign_ins_tests: list[RuleTest] = [
    RuleTest(
        name="Failed Sign-In",
        expected_result=True,
        log={
            "calleripaddress": "12.12.12.12",
            "category": "ServicePrincipalSignInLogs",
            "correlationid": "e1f237ef-6548-4172-be79-03818c04c06e",
            "durationms": 0,
            "level": 4,
            "location": "IE",
            "operationname": "Sign-in activity",
            "operationversion": 1,
            "p_event_time": "2023-07-26 23:00:20.889",
            "p_log_type": "Azure.Audit",
            "properties": {
                "appId": "cfceb902-8fab-4f8c-88ba-374d3c975c3a",
                "authenticationProcessingDetails": [{"key": "Azure AD App Authentication Library", "value": ""}],
                "authenticationProtocol": "none",
                "clientCredentialType": "none",
                "conditionalAccessStatus": "notApplied",
                "correlationId": "5889315c-c4ac-4807-99da-e17417eae786",
                "createdDateTime": "2023-07-26 22:58:30.983201900",
                "crossTenantAccessType": "none",
                "flaggedForReview": False,
                "id": "36658c78-02d9-4d8f-84ee-5ca4a3fdefef",
                "incomingTokenType": "none",
                "ipAddress": "12.12.12.12",
                "isInteractive": False,
                "isTenantRestricted": False,
                "location": {
                    "city": "Dublin",
                    "countryOrRegion": "IE",
                    "geoCoordinates": {"latitude": 51.35555555555555, "longitude": -5.244444444444444},
                    "state": "Dublin",
                },
                "managedIdentityType": "none",
                "processingTimeInMilliseconds": 0,
                "resourceDisplayName": "Azure Storage",
                "resourceId": "037694de-8c7d-498d-917d-edb650090fa5",
                "resourceServicePrincipalId": "a225221f-8cc5-411a-9cc7-5e1394b8a5b8",
                "riskDetail": "none",
                "riskLevelAggregated": "low",
                "riskLevelDuringSignIn": "low",
                "riskState": "none",
                "servicePrincipalId": "b1c34143-e405-4058-8e29-84596ad737b8",
                "servicePrincipalName": "some-service-principal",
                "status": {"errorCode": 7000215},
                "tokenIssuerType": "AzureAD",
                "uniqueTokenIdentifier": "NDDDDDDDDDDDDDDDDDD_DD",
            },
            "resourceid": "/tenants/c0dd2fa0-71be-4df8-b2a6-24cee7de069a/providers/Microsoft.aadiam",
            "resultsignature": "None",
            "resulttype": 7000215,
            "tenantid": "a2aa49aa-2c0c-49d2-af87-f402c421df0b",
            "time": "2023-07-26 23:00:20.889",
        },
    ),
    RuleTest(
        name="Successful Sign-In",
        expected_result=False,
        log={
            "calleripaddress": "12.12.12.12",
            "category": "ServicePrincipalSignInLogs",
            "correlationid": "bf12205b-eea0-43dd-ad6d-b9030dc62a7a",
            "durationms": 0,
            "level": 4,
            "location": "US",
            "operationname": "Sign-in activity",
            "operationversion": 1,
            "p_log_type": "Azure.Audit",
            "properties": {
                "appId": "3b245ca3-dcce-4a54-a070-49ad8de02963",
                "authenticationProcessingDetails": [
                    {
                        "key": "Azure AD App Authentication Library",
                        "value": "Family: Unknown Library: Unknown 1.0.0 Platform: Unknown",
                    },
                ],
                "authenticationProtocol": "none",
                "clientCredentialType": "none",
                "conditionalAccessStatus": "notApplied",
                "correlationId": "52d1e530-786c-443c-8dc8-aa7b1317608e",
                "createdDateTime": "2023-07-27 13:59:53.691680300",
                "crossTenantAccessType": "none",
                "flaggedForReview": False,
                "id": "55270060-d8fe-435e-9bf2-219a1d456b60",
                "incomingTokenType": "none",
                "ipAddress": "12.12.12.12",
                "isInteractive": False,
                "isTenantRestricted": False,
                "location": {
                    "city": "Springfield",
                    "countryOrRegion": "US",
                    "geoCoordinates": {"latitude": 42.73333333333333, "longitude": -110.88888888888889},
                    "state": "Oregon",
                },
                "managedIdentityType": "none",
                "processingTimeInMilliseconds": 0,
                "resourceDisplayName": "Office 365 Management APIs",
                "resourceId": "9cc31481-8822-49ff-b638-552ecc26c777",
                "resourceServicePrincipalId": "2acf8174-5e07-4ec4-9de8-08d880129ba5",
                "riskDetail": "none",
                "riskLevelAggregated": "low",
                "riskLevelDuringSignIn": "low",
                "riskState": "none",
                "servicePrincipalCredentialKeyId": "2afc776a-4e79-4588-b2ad-f62c94d6bea8",
                "servicePrincipalId": "4b6986ec-c49c-40c0-89ce-b2ac51213e39",
                "servicePrincipalName": "very-normal-service-principal",
                "status": {"errorCode": 0},
                "tokenIssuerType": "AzureAD",
                "uniqueTokenIdentifier": "CXXXXXXXXXXXXXXXXXXXXX",
            },
            "resourceid": "/tenants/60641ed1-32f7-4a2e-a912-d724c497e1e9/providers/Microsoft.aadiam",
            "resultsignature": "None",
            "resulttype": 0,
            "tenantid": "85e54ec3-85ee-4b03-9e3b-863075eb9b62",
            "time": "2023-07-27 14:00:41.848",
        },
    ),
]


@panther_managed
class AzureAuditManyFailedSignIns(Rule):
    id = "Azure.Audit.ManyFailedSignIns-prototype"
    display_name = "Azure Many Failed SignIns"
    threshold = 10
    dedup_period_minutes = 10
    log_types = [LogType.AZURE_AUDIT]
    default_severity = Severity.MEDIUM
    default_description = (
        "This detection looks for a number of failed sign-ins for the same ServicePrincipalName or UserPrincipalName\n"
    )
    reports = {"MITRE ATT&CK": ["TA0006:T1110", "TA0001:T1078"]}
    default_runbook = "Querying Sign-In logs for the ServicePrincipalName or UserPrincipalName may indicate that the principal is under attack, or that a sign-in credential rolled and some user of the credential didn't get updated.\n"
    default_reference = "https://learn.microsoft.com/en-us/entra/identity/authentication/overview-authentication"
    summary_attributes = ["properties:ServicePrincipalName", "properties:UserPrincipalName", "properties:ipAddress"]
    tests = azure_audit_many_failed_sign_ins_tests

    def rule(self, event):
        if not is_sign_in_event(event):
            return False
        error_code = deep_get(event, "properties", "status", "errorCode", default=0)
        return error_code > 0

    def title(self, event):
        principal = actor_user(event)
        if principal is None:
            principal = "<NO_PRINCIPALNAME>"
        return f"AzureSignIn: Multiple Failed LogIns for Principal [{principal}]"

    def dedup(self, event):
        principal = actor_user(event)
        if principal is None:
            principal = "<NO_PRINCIPALNAME>"
        return principal

    def alert_context(self, event):
        return azure_signin_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

aws_security_hub_finding_evasion_tests: list[RuleTest] = [
    RuleTest(
        name="CreateInsight",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "3dabcebf-35b0-443f-a1a2-26e186ce23bf",
            "eventName": "CreateInsight",
            "eventSource": "securityhub.amazonaws.com",
            "eventTime": "2018-11-25T01:02:18Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "012345678901",
            "requestID": "c0fffccd-f04d-11e8-93fc-ddcd14710066",
            "requestParameters": {"Filters": {}, "Name": "Test Insight", "ResultField": "ResourceId"},
            "responseElements": {
                "InsightArn": "arn:aws:securityhub:us-west-2:0123456789010:insight/custom/f4c4890b-ac6b-4c26-95f9-e62cc46f3055",
            },
            "sourceIPAddress": "205.251.233.179",
            "userAgent": "aws-cli/1.11.76 Python/2.7.10 Darwin/17.7.0 botocore/1.5.39",
            "userIdentity": {
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "accountId": "012345678901",
                "arn": "arn:aws:iam::012345678901:user/TestUser",
                "principalId": "AIDAJK6U5DS22IAVUI7BW",
                "type": "IAMUser",
                "userName": "TestUser",
            },
        },
    ),
    RuleTest(
        name="DeleteInsight",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "3dabcebf-35b0-443f-a1a2-26e186ce23bf",
            "eventName": "DeleteInsight",
            "eventSource": "securityhub.amazonaws.com",
            "eventTime": "2018-11-25T01:02:18Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "012345678901",
            "requestID": "c0fffccd-f04d-11e8-93fc-ddcd14710066",
            "requestParameters": {"Filters": {}, "Name": "Test Insight", "ResultField": "ResourceId"},
            "responseElements": {
                "InsightArn": "arn:aws:securityhub:us-west-2:0123456789010:insight/custom/f4c4890b-ac6b-4c26-95f9-e62cc46f3055",
            },
            "sourceIPAddress": "205.251.233.179",
            "userAgent": "aws-cli/1.11.76 Python/2.7.10 Darwin/17.7.0 botocore/1.5.39",
            "userIdentity": {
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "accountId": "012345678901",
                "arn": "arn:aws:iam::012345678901:user/TestUser",
                "principalId": "AIDAJK6U5DS22IAVUI7BW",
                "type": "IAMUser",
                "userName": "TestUser",
            },
        },
    ),
    RuleTest(
        name="UpdateFindings",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "3dabcebf-35b0-443f-a1a2-26e186ce23bf",
            "eventName": "UpdateFindings",
            "eventSource": "securityhub.amazonaws.com",
            "eventTime": "2018-11-25T01:02:18Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "012345678901",
            "requestID": "c0fffccd-f04d-11e8-93fc-ddcd14710066",
            "requestParameters": {"Filters": {}, "Name": "Test Insight", "ResultField": "ResourceId"},
            "responseElements": {
                "InsightArn": "arn:aws:securityhub:us-west-2:0123456789010:insight/custom/f4c4890b-ac6b-4c26-95f9-e62cc46f3055",
            },
            "sourceIPAddress": "205.251.233.179",
            "userAgent": "aws-cli/1.11.76 Python/2.7.10 Darwin/17.7.0 botocore/1.5.39",
            "userIdentity": {
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "accountId": "012345678901",
                "arn": "arn:aws:iam::012345678901:user/TestUser",
                "principalId": "AIDAJK6U5DS22IAVUI7BW",
                "type": "IAMUser",
                "userName": "TestUser",
            },
        },
    ),
]


@panther_managed
class AWSSecurityHubFindingEvasion(Rule):
    default_description = "Detections modification of findings in SecurityHub"
    display_name = "AWS SecurityHub Finding Evasion"
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_reference = (
        "https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-insights-view-take-action.html"
    )
    default_severity = Severity.HIGH
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.SecurityHub.Finding.Evasion-prototype"
    tests = aws_security_hub_finding_evasion_tests
    EVASION_OPERATIONS = ["BatchUpdateFindings", "DeleteInsight", "UpdateFindings", "UpdateInsight"]

    def rule(self, event):
        if (
            event.get("eventSource", "") == "securityhub.amazonaws.com"
            and event.get("eventName", "") in self.EVASION_OPERATIONS
        ):
            return True
        return False

    def title(self, event):
        return f"SecurityHub Findings have been modified in account: [{event.get('recipientAccountId', '')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



"""Used to define global helpers for Asana events"""

from pypanther.helpers.panther_base_helpers import deep_get


def asana_alert_context(event: dict) -> dict:
    a_c = {
        "actor": "<NO_ACTOR>",
        "context": "<NO_CONTEXT>",
        "event_type": "<NO_EVENT_TYPE>",
        "resource_type": "<NO_RESOURCE_TYPE>",
        "resource_name": "<NO_RESOURCE_NAME>",
        "resource_gid": "<NO_RESOURCE_GID>",
    }
    if deep_get(event, "actor", "actor_type", default="") == "user":
        a_c["actor"] = deep_get(event, "actor", "email", default="<NO_ACTOR_EMAIL>")
    else:
        a_c["actor"] = deep_get(event, "actor", "actor_type", default="<NO_ACTOR>")
    if "event_type" in event:
        # Events have categories and event_type
        # We have not seen category overlap -> only including event_type
        a_c["event_type"] = event.get("event_type")
    a_c["resource_name"] = deep_get(event, "resource", "name", default="<NO_RESOURCE_NAME>")
    a_c["resource_gid"] = deep_get(event, "resource", "gid", default="<NO_RESOURCE_GID>")
    r_t = deep_get(event, "resource", "resource_type")
    if r_t:
        a_c["resource_type"] = r_t
    r_s_t = deep_get(event, "resource", "resource_subtype")
    if r_t and r_s_t and r_s_t != r_t:
        a_c["resource_type"] += "__" + r_s_t
    ctx = deep_get(event, "context", "context_type")
    if ctx:
        a_c["context"] = ctx
    return a_c
